local Tinkr, _Bastion = ...

---@class Bastion
local Bastion = _Bastion

---@alias BigWigsBars table<string, { spellId: number, duration: number, startTime: number, pauseTime: number | nil }>

---@class MythicPlusUtils
local MythicPlusUtils = {
    ---@type InterruptList
    interruptList = {},
    ---@type TankBusterList
    tankBusterList = {},
    ---@type BigWigsBars
    bars = {},
    isBigWigsEnabled = false
}

MythicPlusUtils.__index = MythicPlusUtils

local function BWEventCallBack(event, ...)
    if event == "BigWigs_StartBar" then
        local addon, spellId, text, duration, icon, isCD = ...
        local now = GetTime()

        print('START BAR', text)

        MythicPlusUtils.bars[text] = MythicPlusUtils.bars[text] or {}

        MythicPlusUtils.bars[text].duration = duration
        MythicPlusUtils.bars[text].startTime = now
        MythicPlusUtils.bars[text].spellId = spellId
    elseif event == "BigWigs_StopBar" then
        local addon, text = ...

        print('STOP BAR', text)

        if MythicPlusUtils.bars[text] then
            MythicPlusUtils.bars[text] = nil
        end
    elseif event == "BigWigs_PauseBar" then
        local addon, text = ...

        print('PAUSE BAR', text)

        if MythicPlusUtils.bars[text] then
            MythicPlusUtils.bars[text].pauseTime = GetTime()
        end
    elseif event == "BigWigs_ResumeBar" then
        local addon, text = ...

        print('RESUME BAR', text)

        if MythicPlusUtils.bars[text] and MythicPlusUtils.bars[text].pauseTime then
            local pauseTime = MythicPlusUtils.bars[text].pauseTime
            local startTime = MythicPlusUtils.bars[text].startTime
            local duration = MythicPlusUtils.bars[text].duration

            local newDuration = duration - (pauseTime - startTime)

            MythicPlusUtils.bars[text].duration = newDuration
            MythicPlusUtils.bars[text].startTime = GetTime()
            MythicPlusUtils.bars[text].pauseTime = nil
        end
    elseif event == "BigWigs_StopBars" or event == "BigWigs_OnBossDisable" then
        MythicPlusUtils.bars = {}
    end
end

---@type InterruptList
local _interruptList = Tinkr:require("scripts/bastion/MythicPlusUtils/lists/interrupt-list", Bastion)
---@type TankBusterList
local _tankBusterList = Tinkr:require("scripts/bastion/MythicPlusUtils/lists/tank-buster-list", Bastion)

---@return MythicPlusUtils
function MythicPlusUtils:New()
    local self = setmetatable({}, MythicPlusUtils)

    self.interruptList = _interruptList
    self.tankBusterList = _tankBusterList

    if BigWigsLoader then
        print("BigWigs Enabled!")
        self.isBigWigsEnabled = true
        BigWigsLoader.RegisterMessage({}, 'BigWigs_StartBar', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_StopBar', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_PauseBar', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_ResumeBar', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_StopBars', BWEventCallBack)
        BigWigsLoader.RegisterMessage({}, 'BigWigs_OnBossDisable', BWEventCallBack)
    end

    return self
end

---@param unit Unit
---@param percent number
---@return { kickable: boolean, paraliseable: boolean, sweepable: boolean } | nil
function MythicPlusUtils:CastingCriticalStop(unit, percent)
    local castingSpell = unit:GetCastingOrChannelingSpell()

    local npcSpells = self.interruptList[unit:GetID()]

    if npcSpells and castingSpell then
        local spellID = castingSpell:GetID()

        local spellTraits = npcSpells[spellID]

        if not spellTraits then
            return nil
        end

        local castPercent = unit:GetChannelOrCastPercentComplete()

        if castPercent >= percent then
            return spellTraits
        end
    end

    return nil
end

function MythicPlusUtils:RefreshBars()
    if self.isBigWigsEnabled then
        for key, value in pairs(self.bars) do
            local expireTime = value.duration + value.startTime

            if expireTime < GetTime() then
                self.bars[key] = nil
            end
        end
    end
end

---@param spellId number
---@param timeUntilCast number
---@return boolean
function MythicPlusUtils:IsSpellCastSoon(spellId, timeUntilCast)
    if self.isBigWigsEnabled then
        self:RefreshBars()
        for key, value in pairs(self.bars) do
            if value.spellId == spellId then
                local expireTime = value.duration + value.startTime

                local timeUntilSpell = expireTime - GetTime()
    
                if timeUntilSpell > 0 and timeUntilSpell <= timeUntilCast then
                    return true
                end
            end
        end
    end
    return false
end

---@param unit Unit
---@param timeUntilCast? number
---@return boolean
function MythicPlusUtils:CastingTankBuster(unit, timeUntilCast)

    return false
end

return MythicPlusUtils
